# Copyright 2008-2009 (C) Pascal Bach <pascal.bach@nextrem.ch>
#
#
# Based on work from
# Copyright 2007 (C) Brian Sipos <bjs5075@rit.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

"""Prepares a Pro/E project directory to be stored into Bazaar.

The aim of this Project is to provide a set of tools to use the Bazaar VCS to handle PTC Pro Engineer CAD files.

It currently has a utility (bzr proclean) to clean up and normalise a Pro/E Working directory to prepare it to be stored in Bazaar.

"""


import sys
import os
import shutil
import re

from bzrlib.commands import Command, register_command
from bzrlib.option import Option

version_info = (0,1,0)
plugin_name = 'proe'

# RE matching ProE files
# .prt = Part
# .asm = Assembly
# .drw = Drawing
# .sec = Sketch
proeFileRegex = re.compile('(.*\.(prt|asm|drw|sec|))(\.\d+)?$')
trailFileRegex = re.compile('.*trail\.txt(\.\d+)?')

def str2rev(s):
    """ Convert a revision string into number """
    if not s:
        return 0
    return int(s[1:])

def rev2str(r):
    """ Convert a revision number into string """
    if r == 0:
        return ''
    return '.' + str(r)


def removeTrailFiles(curr_dir,files,dry_run=False):
    """ Remove ProE Trail files """
    for f in files:
        path = os.path.join(curr_dir,f)
        if os.path.isfile(path) and trailFileRegex.match(path):
            print "Removing:", path
            if not dry_run:
                os.remove(path)
            
def removeProeFiles(curr_dir,files,dry_run=False):
    """ Remove and Nomrmalise ProE files """
    proeFileList = {}
    for f in files:
        path = os.path.join(curr_dir,f)
        match = proeFileRegex.match(path)
        if os.path.isfile(path) and match:
            name, ext, rev = match.groups()
            rev = str2rev(rev)
            
            # Append the name to the list
            if not name in proeFileList:
                proeFileList[name] = []
            
            proeFileList[name].append(rev)
            
            
    # Traverse the list of files, deleting all but the maximum revision
    # and renaming the maximum to have no extension            
    for name in proeFileList.keys():
        
        print "Normalizing:", name
        if not dry_run:
            proeFileList[name].sort()
    
            latestRev = proeFileList[name].pop()
            
            for oldRev in proeFileList[name]:
                os.remove(name + rev2str(oldRev))
    
            shutil.move(name + rev2str(latestRev), name)
        
            
class cmd_proclean(Command):
    """ Clean up and normalises a ProE project direcotry """
    takes_options = [
        Option('dry-run', help="Show what would be done, but don't actually do anything."),
        ]
    def run(self, dry_run=False):
        dir0 = os.getcwd()
        if dry_run:
            print "Dry Run!"
        for root, dirs, files in os.walk(dir0):
            removeTrailFiles(root,files,dry_run)
            removeProeFiles(root,files,dry_run)

register_command(cmd_proclean) 
